@@ -79,10 +79,8 @@ module Haml
           @options[:trace] = true
         end
 
-        if ::Haml::Util.windows?
-          opts.on('--unix-newlines', 'Use Unix-style newlines in written files.') do
-            @options[:unix_newlines] = true
-          end
+        opts.on('--unix-newlines', 'Use Unix-style newlines in written files.') do
+          @options[:unix_newlines] = true if ::Haml::Util.windows?
         end
 
         opts.on_tail("-?", "-h", "--help", "Show this message") do
@@ -126,9 +124,18 @@ module Haml
       # @param color [Symbol] The name of the color to use for this action.
       #   Can be `:red`, `:green`, or `:yellow`.
       def puts_action(name, color, arg)
+        return if @options[:for_engine][:quiet]
         printf color(color, "%11s %s\n"), name, arg
       end
 
+      # Same as \{Kernel.puts}, but doesn't print anything if the `--quiet` option is set.
+      #
+      # @param args [Array] Passed on to \{Kernel.puts}
+      def puts(*args)
+        return if @options[:for_engine][:quiet]
+        Kernel.puts(*args)
+      end
+
       # Wraps the given string in terminal escapes
       # causing it to have the given color.
       # If terminal esapes aren't supported on this platform,
@@ -157,11 +164,11 @@ module Haml
       end
 
       def handle_load_error(err)
-        dep = err.message.scan(/^no such file to load -- (.*)/)[0]
+        dep = err.message[/^no such file to load -- (.*)/, 1]
         raise err if @options[:trace] || dep.nil? || dep.empty?
         $stderr.puts <<MESSAGE
 Required dependency #{dep} not found!
-  Run "gem install #{dep}" to get it.
+    Run "gem install #{dep}" to get it.
   Use --trace for backtrace.
 MESSAGE
         exit 1
@@ -207,7 +214,7 @@ END
               "haml --rails will no longer work in the next version of #{@name}.", "")
           elsif File.exists?(env) && File.open(env) {|env| env.grep(/config\.gem/)}
             puts("haml --rails isn't needed for Rails 2.1 or greater.",
-              "Add 'gem \"haml\"' to config/environment.rb instead.", "",
+              "Add 'config.gem \"haml\"' to config/environment.rb instead.", "",
               "haml --rails will no longer work in the next version of #{@name}.", "")
           end
 
@@ -294,11 +301,15 @@ END
                             'Locations are set like --watch.') do
           @options[:update] = true
         end
+        opts.on('--stop-on-error', 'If a file fails to compile, exit immediately.',
+                                   'Only meaningful for --watch and --update.') do
+          @options[:stop_on_error] = true
+        end
         opts.on('-t', '--style NAME',
                 'Output style. Can be nested (default), compact, compressed, or expanded.') do |name|
           @options[:for_engine][:style] = name.to_sym
         end
-        opts.on('-q', '--quiet', 'Silence warnings during compilation.') do
+        opts.on('-q', '--quiet', 'Silence warnings and status messages during compilation.') do
           @options[:for_engine][:quiet] = true
         end
         opts.on('-g', '--debug-info',
@@ -353,7 +364,7 @@ END
           input = @options[:input]
           output = @options[:output]
 
-          @options[:syntax] ||= :scss if input.is_a?(File) && input.path =~ /\.scss$/
+          @options[:for_engine][:syntax] ||= :scss if input.is_a?(File) && input.path =~ /\.scss$/
           tree =
             if input.is_a?(File) && !@options[:check_syntax]
               ::Sass::Files.tree_for(input.path, @options[:for_engine])
@@ -388,7 +399,13 @@ END
         ::Sass::Plugin.options.merge! @options[:for_engine]
         ::Sass::Plugin.options[:unix_newlines] = @options[:unix_newlines]
 
-        if @args[1] && !colon_path?(@args[0])
+        raise <<MSG if @args.empty?
+What files should I watch? Did you mean something like:
+    sass --watch input.sass:output.css
+    sass --watch input-dir:output-dir
+MSG
+
+        if !colon_path?(@args[0]) && probably_dest_dir?(@args[1])
           flag = @options[:update] ? "--update" : "--watch"
           err =
             if !File.exist?(@args[1])
@@ -398,7 +415,7 @@ END
             end
           raise <<MSG if err
 File #{@args[1]} #{err}.
-  Did you mean: sass #{flag} #{@args[0]}:#{@args[1]}
+    Did you mean: sass #{flag} #{@args[0]}:#{@args[1]}
 MSG
         end
 
@@ -416,15 +433,18 @@ MSG
           end
         end
 
+        had_error = false
         ::Sass::Plugin.on_creating_directory {|dirname| puts_action :directory, :green, dirname}
         ::Sass::Plugin.on_deleting_css {|filename| puts_action :delete, :yellow, filename}
         ::Sass::Plugin.on_compilation_error do |error, _, _|
-          raise error unless error.is_a?(::Sass::SyntaxError)
+          raise error unless error.is_a?(::Sass::SyntaxError) && !@options[:stop_on_error]
+          had_error = true
           puts_action :error, :red, "#{error.sass_filename} (Line #{error.sass_line}: #{error.message})"
         end
 
         if @options[:update]
           ::Sass::Plugin.update_stylesheets(files)
+          exit 1 if had_error
           return
         end
 
@@ -452,6 +472,14 @@ MSG
         end
         return one, two
       end
+
+      # Whether path is likely to be meant as the destination
+      # in a source:dest pair.
+      def probably_dest_dir?(path)
+        return false unless path
+        return false if colon_path?(path)
+        return Dir.glob(File.join(path, "*.s[ca]ss")).empty?
+      end
     end
 
     # The `haml` executable.
