@@ -98,7 +98,8 @@ module Sass
         node << comment
       end
 
-      DIRECTIVES = Set[:mixin, :include, :debug, :warn, :for, :while, :if, :extend, :import, :media]
+      DIRECTIVES = Set[:mixin, :include, :debug, :warn, :for, :while, :if, :else,
+        :extend, :import, :media, :charset]
 
       def directive
         return unless tok(/@/)
@@ -109,11 +110,10 @@ module Sass
           return dir
         end
 
-        val = str do
-          # Most at-rules take expressions (e.g. @import),
-          # but some (e.g. @page) take selector-like arguments
-          expr || selector
-        end
+        # Most at-rules take expressions (e.g. @import),
+        # but some (e.g. @page) take selector-like arguments
+        val = str {break unless expr}
+        val ||= CssParser.new(@scanner, @line).parse_selector_string
         node = node(Sass::Tree::DirectiveNode.new("@#{name} #{val}".strip))
 
         if tok(/\{/)
@@ -180,12 +180,14 @@ module Sass
         ss
         node = block(node(Sass::Tree::IfNode.new(expr)), :directive)
         pos = @scanner.pos
+        line = @line
         ss
 
         else_block(node) ||
           begin
             # Backtrack in case there are any comments we want to parse
             @scanner.pos = pos
+            @line = line
             node
           end
       end
@@ -198,23 +200,40 @@ module Sass
           :directive)
         node.add_else(else_node)
         pos = @scanner.pos
+        line = @line
         ss
 
         else_block(node) ||
           begin
             # Backtrack in case there are any comments we want to parse
             @scanner.pos = pos
+            @line = line
             node
           end
       end
 
+      def else_directive
+        raise Sass::SyntaxError.new(
+          "Invalid CSS: @else must come after @if", :line => @line)
+      end
+
       def extend_directive
         node(Sass::Tree::ExtendNode.new(expr!(:selector)))
       end
 
       def import_directive
-        @expected = "string or url()"
-        arg = tok(STRING) || (uri = tok!(URI))
+        values = []
+
+        loop do
+          values << expr!(:import_arg)
+          break if use_css_import? || !tok(/,\s*/)
+        end
+
+        return values
+      end
+
+      def import_arg
+        return unless arg = tok(STRING) || (uri = tok!(URI))
         path = @scanner[1] || @scanner[2] || @scanner[3]
         ss
 
@@ -280,6 +299,13 @@ module Sass
         true
       end
 
+      def charset_directive
+        tok! STRING
+        name = @scanner[1] || @scanner[2]
+        ss
+        node(Sass::Tree::CharsetNode.new(name))
+      end
+
       def variable
         return unless tok(/\$/)
         name = tok!(IDENT)
@@ -319,7 +345,7 @@ module Sass
       def block_contents(node, context)
         block_given? ? yield : ss_comments(node)
         node << (child = block_child(context))
-        while tok(/;/) || (child && child.has_children)
+        while tok(/;/) || has_children?(child)
           block_given? ? yield : ss_comments(node)
           node << (child = block_child(context))
         end
@@ -331,6 +357,12 @@ module Sass
         variable || directive || declaration_or_ruleset
       end
 
+      def has_children?(child_or_array)
+        return false unless child_or_array
+        return child_or_array.last.has_children if child_or_array.is_a?(Array)
+        return child_or_array.has_children
+      end
+
       # This is a nasty hack, and the only place in the parser
       # that requires backtracking.
       # The reason is that we can't figure out if certain strings
@@ -644,11 +676,10 @@ MESSAGE
         unless e = tok(NUMBER) ||
             tok(URI) ||
             function ||
-            interp_string ||
+            tok(STRING) ||
             tok(UNICODERANGE) ||
             tok(IDENT) ||
-            tok(HEXCOLOR) ||
-            interpolation
+            tok(HEXCOLOR)
 
           return unless op = unary_operator
           @expected = "number or function"
@@ -749,6 +780,7 @@ MESSAGE
         :expr => "expression (e.g. 1px, bold)",
         :selector_comma_sequence => "selector",
         :simple_selector_sequence => "selector",
+        :import_arg => "file to import (string or url())",
       }
 
       TOK_NAMES = Haml::Util.to_hash(
