@@ -84,14 +84,18 @@ module Sass
 
       # Appends a child to the node.
       #
-      # @param child [Tree::Node] The child node
+      # @param child [Tree::Node, Array<Tree::Node>] The child node or nodes
       # @raise [Sass::SyntaxError] if `child` is invalid
       # @see #invalid_child?
       def <<(child)
         return if child.nil?
-        check_child! child
-        self.has_children = true
-        @children << child
+        if child.is_a?(Array)
+          child.each {|c| self << c}
+        else
+          check_child! child
+          self.has_children = true
+          @children << child
+        end
       end
 
       # Raises an error if the given child node is invalid.
@@ -311,6 +315,7 @@ module Sass
       # @see #perform
       def perform!(environment)
         self.children = perform_children(Environment.new(environment))
+        self.children.each {|c| check_child! c}
       end
 
       # Non-destructively runs \{#perform} on all children of the current node.
@@ -364,8 +369,6 @@ module Sass
           "Mixins may only be defined at the root of a document."
         when Tree::ImportNode
           "Import directives may only be used at the root of a document."
-        when Tree::ExtendNode
-          "Extend directives may only be used within rules."
         end
       end
 
