@@ -114,6 +114,10 @@ MSG
     '@warn' => "Invalid warn directive '@warn': expected expression.",
     %Q{@warn "a message"\n  "nested message"} => "Illegal nesting: Nothing may be nested beneath warn directives.",
     "/* foo\n    bar\n  baz" => "Inconsistent indentation: previous line was indented by 4 spaces, but this line was indented by 2 spaces.",
+    "@if foo\n  @extend .bar" => ["Extend directives may only be used within rules.", 2],
+    "$var: true\n@while $var\n  @extend .bar\n  $var: false" => ["Extend directives may only be used within rules.", 3],
+    "@for $i from 0 to 1\n  @extend .bar" => ["Extend directives may only be used within rules.", 2],
+    "@mixin foo\n  @extend .bar\n@include foo" => ["Extend directives may only be used within rules.", 2],
 
     # Regression tests
     "a\n  b:\n    c\n    d" => ["Illegal nesting: Only properties may be nested beneath properties.", 3],
@@ -490,17 +494,6 @@ CSS
     assert_equal("@import url(./fonts.css);\n", render("@import \"./fonts.css\""))
   end
 
-  def test_media_import
-    assert_equal("@import \"./fonts.sass\" all;\n",
-      render("@import \"./fonts.sass\" all"))
-    assert_equal(<<CSS, render(<<SASS))
-@import "./fonts.sass" all;
-@import url(./fonts.scss);
-CSS
-@import "./fonts.sass" all, url(./fonts.scss)
-SASS
-  end
-
   def test_http_import
     assert_equal("@import url(http://fonts.googleapis.com/css?family=Droid+Sans);\n",
       render("@import \"http://fonts.googleapis.com/css?family=Droid+Sans\""))
@@ -1484,6 +1477,32 @@ foo
 SASS
   end
 
+  def test_loud_comment_with_close
+    assert_equal <<CSS, render(<<SASS)
+foo {
+  /* foo
+   * bar */ }
+CSS
+foo
+  /* foo
+     bar */
+SASS
+  end
+
+  def test_loud_comment_with_separate_line_close
+    assert_equal <<CSS, render(<<SASS)
+foo {
+  /* foo
+   * bar
+   */ }
+CSS
+foo
+  /* foo
+   * bar
+   */
+SASS
+  end
+
   def test_attribute_selector_with_spaces
     assert_equal(<<CSS, render(<<SASS))
 a b[foo=bar] {
@@ -2042,7 +2061,7 @@ SASS
   end
 
   def test_mixin_no_arg_error
-    assert_raise(Sass::SyntaxError, 'Invalid CSS after "($bar,": expected variable name, was ")"') do
+    assert_raise_message(Sass::SyntaxError, 'Invalid CSS after "($bar,": expected variable (e.g. $foo), was ")"') do
       render(<<SASS)
 =foo($bar,)
   bip: bap
@@ -2058,6 +2077,38 @@ CSS
 SASS
   end
 
+  def test_function_output_with_comma
+    assert_equal <<CSS, render(<<SASS)
+foo {
+  a: b(c), d(e); }
+CSS
+foo
+  a: b(c), d(e)
+SASS
+  end
+
+  def test_interpolation_with_comma
+    assert_equal <<CSS, render(<<SASS)
+foo {
+  a: foo, bar; }
+CSS
+$foo: foo
+foo
+  a: \#{$foo}, bar
+SASS
+  end
+
+  def test_string_interpolation_with_comma
+    assert_equal <<CSS, render(<<SASS)
+foo {
+  a: "bip foo bap", bar; }
+CSS
+$foo: foo
+foo
+  a: "bip \#{$foo} bap", bar
+SASS
+  end
+
   # Encodings
 
   unless Haml::Util.ruby1_8?
@@ -2081,7 +2132,7 @@ SASS
 
     def test_same_charset_as_encoding
       assert_renders_encoded(<<CSS, <<SASS)
-@charset "utf-8";
+@charset "UTF-8";
 fóó {
   a: b; }
 CSS
@@ -2093,7 +2144,7 @@ SASS
 
     def test_different_charset_than_encoding
       assert_renders_encoded(<<CSS.force_encoding("IBM866"), <<SASS)
-@charset "ibm866";
+@charset "IBM866";
 fóó {
   a: b; }
 CSS
@@ -2105,6 +2156,7 @@ SASS
 
     def test_different_encoding_than_system
       assert_renders_encoded(<<CSS.encode("IBM866"), <<SASS.encode("IBM866"))
+@charset "IBM866";
 тАЬ {
   a: b; }
 CSS
@@ -2114,20 +2166,20 @@ SASS
     end
 
     def test_multibyte_charset
-      assert_renders_encoded(<<CSS.encode("UTF-16BE"), <<SASS.encode("UTF-16BE").force_encoding("UTF-8"))
-@charset "utf-16be";
+      assert_renders_encoded(<<CSS.encode("UTF-16LE"), <<SASS.encode("UTF-16LE").force_encoding("UTF-8"))
+@charset "UTF-16LE";
 fóó {
   a: b; }
 CSS
-@charset "utf-16be"
+@charset "utf-16le"
 fóó
   a: b
 SASS
     end
 
     def test_multibyte_charset_without_endian_specifier
-      assert_renders_encoded(<<CSS.encode("UTF-32LE"), <<SASS.encode("UTF-32LE").force_encoding("UTF-8"))
-@charset "utf-32";
+      assert_renders_encoded(<<CSS.encode("UTF-32BE"), <<SASS.encode("UTF-32BE").force_encoding("UTF-8"))
+@charset "UTF-32BE";
 fóó {
   a: b; }
 CSS
@@ -2139,6 +2191,7 @@ SASS
 
     def test_utf8_bom
       assert_renders_encoded(<<CSS, <<SASS.force_encoding("BINARY"))
+@charset "UTF-8";
 fóó {
   a: b; }
 CSS
@@ -2149,6 +2202,7 @@ SASS
 
     def test_utf16le_bom
       assert_renders_encoded(<<CSS.encode("UTF-16LE"), <<SASS.encode("UTF-16LE").force_encoding("BINARY"))
+@charset "UTF-16LE";
 fóó {
   a: b; }
 CSS
@@ -2159,6 +2213,7 @@ SASS
 
     def test_utf32be_bom
       assert_renders_encoded(<<CSS.encode("UTF-32BE"), <<SASS.encode("UTF-32BE").force_encoding("BINARY"))
+@charset "UTF-32BE";
 fóó {
   a: b; }
 CSS
