@@ -13,13 +13,13 @@ class SassScriptTest < Test::Unit::TestCase
   include Sass::Script
 
   def test_color_checks_input
-    assert_raise(Sass::SyntaxError, "Color values must be between 0 and 255") {Color.new([1, 2, -1])}
-    assert_raise(Sass::SyntaxError, "Color values must be between 0 and 255") {Color.new([256, 2, 3])}
+    assert_raise_message(Sass::SyntaxError, "Blue value must be between 0 and 255") {Color.new([1, 2, -1])}
+    assert_raise_message(Sass::SyntaxError, "Red value must be between 0 and 255") {Color.new([256, 2, 3])}
   end
 
   def test_color_checks_rgba_input
-    assert_raise(Sass::SyntaxError, "Alpha channel must be between 0 and 1") {Color.new([1, 2, 3, 1.1])}
-    assert_raise(Sass::SyntaxError, "Alpha channel must be between 0 and 1") {Color.new([1, 2, 3, -0.1])}
+    assert_raise_message(Sass::SyntaxError, "Alpha channel must be between 0 and 1") {Color.new([1, 2, 3, 1.1])}
+    assert_raise_message(Sass::SyntaxError, "Alpha channel must be between 0 and 1") {Color.new([1, 2, 3, -0.1])}
   end
 
   def test_string_escapes
@@ -63,13 +63,13 @@ class SassScriptTest < Test::Unit::TestCase
     assert_equal "rgba(50, 50, 100, 0.35)", resolve("rgba(1, 1, 2, 0.35) * rgba(50, 50, 50, 0.35)")
     assert_equal "rgba(52, 52, 52, 0.25)", resolve("rgba(2, 2, 2, 0.25) + rgba(50, 50, 50, 0.25)")
 
-    assert_raise(Sass::SyntaxError, "Alpha channels must be equal: rgba(1, 2, 3, 0.15) + rgba(50, 50, 50, 0.75)") do
+    assert_raise_message(Sass::SyntaxError, "Alpha channels must be equal: rgba(1, 2, 3, 0.15) + rgba(50, 50, 50, 0.75)") do
       resolve("rgba(1, 2, 3, 0.15) + rgba(50, 50, 50, 0.75)")
     end
-    assert_raise(Sass::SyntaxError, "Alpha channels must be equal: #123456 * rgba(50, 50, 50, 0.75)") do
+    assert_raise_message(Sass::SyntaxError, "Alpha channels must be equal: #123456 * rgba(50, 50, 50, 0.75)") do
       resolve("#123456 * rgba(50, 50, 50, 0.75)")
     end
-    assert_raise(Sass::SyntaxError, "Alpha channels must be equal: #123456 / #123456") do
+    assert_raise_message(Sass::SyntaxError, "Alpha channels must be equal: rgba(50, 50, 50, 0.75) / #123456") do
       resolve("rgba(50, 50, 50, 0.75) / #123456")
     end
   end
@@ -79,9 +79,13 @@ class SassScriptTest < Test::Unit::TestCase
     assert_equal "rgba(100, 100, 100, 0.75)", resolve("rgba(50, 50, 50, 0.75) * 2")
   end
 
+  def test_rgba_rounding
+    assert_equal "rgba(10, 1, 0, 0.123)", resolve("rgba(10.0, 1.23456789, 0.0, 0.1234567)")
+  end
+
   def test_compressed_colors
     assert_equal "#123456", resolve("#123456", :style => :compressed)
-    assert_equal "rgba(1, 2, 3, 0.5)", resolve("rgba(1, 2, 3, 0.5)", :style => :compressed)
+    assert_equal "rgba(1,2,3,0.5)", resolve("rgba(1, 2, 3, 0.5)", :style => :compressed)
     assert_equal "#123", resolve("#112233", :style => :compressed)
     assert_equal "#000", resolve("black", :style => :compressed)
     assert_equal "red", resolve("#f00", :style => :compressed)
@@ -91,6 +95,12 @@ class SassScriptTest < Test::Unit::TestCase
     assert_equal "This color is #fff", resolve('"This color is #{ white }"', :style => :compressed)
   end
 
+  def test_compressed_comma
+    # assert_equal "foo,bar,baz", resolve("foo, bar, baz", :style => :compressed)
+    # assert_equal "foo,#baf,baz", resolve("foo, #baf, baz", :style => :compressed)
+    assert_equal "foo,#baf,red", resolve("foo, #baf, #f00", :style => :compressed)
+  end
+
   def test_implicit_strings
     assert_equal Sass::Script::String.new("foo"), eval("foo")
     assert_equal Sass::Script::String.new("foo bar"), eval("foo bar")
@@ -360,18 +370,24 @@ SASS
   end
 
   def test_colors_with_wrong_number_of_digits
-    assert_raise(Sass::SyntaxError,
+    assert_raise_message(Sass::SyntaxError,
       "Colors must have either three or six digits: '#0'") {eval("#0")}
-    assert_raise(Sass::SyntaxError,
+    assert_raise_message(Sass::SyntaxError,
       "Colors must have either three or six digits: '#12'") {eval("#12")}
-    assert_raise(Sass::SyntaxError,
+    assert_raise_message(Sass::SyntaxError,
       "Colors must have either three or six digits: '#abcd'") {eval("#abcd")}
-    assert_raise(Sass::SyntaxError,
+    assert_raise_message(Sass::SyntaxError,
       "Colors must have either three or six digits: '#abcdE'") {eval("#abcdE")}
-    assert_raise(Sass::SyntaxError,
+    assert_raise_message(Sass::SyntaxError,
       "Colors must have either three or six digits: '#abcdEFA'") {eval("#abcdEFA")}
   end
 
+  def test_case_insensitive_color_names
+    assert_equal "blue", resolve("BLUE")
+    assert_equal "red", resolve("rEd")
+    assert_equal "#7f4000", resolve("mix(GrEeN, ReD)")
+  end
+
   # Regression Tests
 
   def test_funcall_has_higher_precedence_than_color_name
@@ -386,7 +402,7 @@ SASS
   end
 
   def test_misplaced_comma_in_funcall
-    assert_raise(Sass::SyntaxError,
+    assert_raise_message(Sass::SyntaxError,
       'Invalid CSS after "foo(bar, ": expected function argument, was ")"') {eval('foo(bar, )')}
   end
 
@@ -426,6 +442,7 @@ SASS
 
   def eval(str, opts = {}, environment = env)
     munge_filename opts
+    environment.options = opts
     Sass::Script.parse(str, opts.delete(:line) || 1,
       opts.delete(:offset) || 0, opts).perform(environment)
   end
