@@ -49,11 +49,34 @@ baz {bar: baz}
 SCSS
   end
 
-  def test_unicode
-    assert_parses <<SCSS
+  if Haml::Util.ruby1_8?
+    def test_unicode
+      assert_parses <<SCSS
+@charset "UTF-8";
 foo {
   bar: föö bâr; }
 SCSS
+      assert_parses <<SCSS
+foo {
+  bar: föö bâr; }
+SCSS
+    end
+  else
+    def test_unicode
+      assert_parses <<SCSS
+@charset "UTF-8";
+foo {
+  bar: föö bâr; }
+SCSS
+      assert_equal <<CSS, render(<<SCSS)
+@charset "UTF-8";
+foo {
+  bar: föö bâr; }
+CSS
+foo {
+  bar: föö bâr; }
+SCSS
+    end
   end
 
   def test_invisible_comments
@@ -393,7 +416,7 @@ SCSS
     assert_equal <<CSS, render(<<SCSS)
 foo {
   a: -0.5em;
-  b: +0.5em;
+  b: 0.5em;
   c: -foo(12px);
   d: +foo(12px); }
 CSS
@@ -429,10 +452,6 @@ SCSS
 
   ## Directives
 
-  def test_charset_directive
-    assert_parses '@charset "utf-8";'
-  end
-
   def test_namespace_directive
     assert_parses '@namespace "http://www.w3.org/Profiles/xhtml1-strict";'
     assert_parses '@namespace url(http://www.w3.org/Profiles/xhtml1-strict);'
@@ -508,11 +527,11 @@ SCSS
   end
 
   def test_blockless_directive_without_semicolon
-    assert_equal "@charset \"utf-8\";\n", render('@charset "utf-8"')
+    assert_equal "@foo \"bar\";\n", render('@foo "bar"')
   end
 
   def test_directive_with_lots_of_whitespace
-    assert_equal "@charset \"utf-16\";\n", render('@charset    "utf-16"  ;')
+    assert_equal "@foo \"bar\";\n", render('@foo    "bar"  ;')
   end
 
   def test_empty_blockless_directive
@@ -866,6 +885,20 @@ SCSS
     assert_equal 1, e.sass_line
   end
 
+  ## Regressions
+
+  def test_closing_line_comment_end_with_compact_output
+    assert_equal(<<CSS, render(<<SCSS, :style => :compact))
+/* foo */
+bar { baz: bang; }
+CSS
+/*
+ * foo
+ */
+bar {baz: bang}
+SCSS
+  end
+
   private
 
   def assert_selector_parses(selector)
